import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Scanner;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * A class for launching simulations of different SpacePort models
 * 
 * @author Joel Ross
 * @version Sp14
 */
public class SpacePortSimulation
{
	public enum Statistic {CURRENT_DOCKED, MAX_DOCKED, BERTHS, 
		AVG_COLLISIONS_SUCCESSFUL, MAX_COLLISIONS_SUCCESSFUL,
		AVG_COLLISIONS_UNSUCCESSFUL, MAX_COLLISIONS_UNSUCCESSFUL,
		AVG_COLLISIONS_ALL, MAX_COLLISIONS_ALL,
	};
	
	public static void main(String[] args)
	{
		/* Pick the hash function to use */
		LicenseHasher hasher = new DefaultHasher();
		//LicenseHasher hasher = new MyHasher(); //for example; whatever kind of hasher you want!
		
		/* Pick the hash map version to use */
		double loadThreshold = 0.75;
		SpacePort spacePort = new BasicSpacePort(hasher, loadThreshold);
		//SpacePort spacePort = new DimensionalSpacePort(hasher);
		
		/* run the simulation */
		File simulationFile = getFile("simulation1.txt"); //fetch the specified simulation
		simulateSpacePort(spacePort, simulationFile);
	}
	
	
	/**
	 * Simulates usage of the given SpacePort using details found in the given simulation file.
	 * Prints out statistics of the usage at the end.
	 * @param spacePort The SpacePort to simulate with
	 * @param simulationFile A file containing details about usage of the SpacePort, specifying actions to perform in the simulation
	 */
	public static void simulateSpacePort(SpacePort spacePort, File simulationFile)
	{
		try {
			Scanner sc = new Scanner(simulationFile); //scan the file for instructions
			while(sc.hasNextLine()) //while more to do
			{
				String[] parts = sc.nextLine().split(","); //get the pieces of the next line

				//check what to do!
				if(parts[0].equals("Arrive")) //arrival
				{
					String license = parts[1]; //information about the ship involved in this event
					String cargo = parts[2];
					SpaceShip arrival = new SpaceShip(license, cargo);
					System.out.println("Now arriving: "+arrival);
					boolean successful = spacePort.dock(license, arrival);
					if(!successful) //error handling...
					{
						System.out.println("*** ALERT ***: Vessel with forged license detected. Detaining...");
						spacePort.remove(license); //impound the ship
					}
				}
				else if(parts[0].equals("Depart")) //departure
				{
					String license = parts[1]; //information about the ship involved in this event
					System.out.println("Departure clearance granted to vessel "+license);
					SpaceShip departure = spacePort.remove(license); //the ship departs
					if(departure == null)
						System.out.println("*** ALERT ***: Departure clearance granted to non-existent vessel!"); //someone's getting fired
				}
				else if(parts[0].equals("Inspect")) //inspection
				{
					String license = parts[1]; //information about the ship involved in this event
					System.out.println("Inspecting docked ship "+license);
					SpaceShip inspected = spacePort.access(license);
					if(inspected == null)
						System.out.println("*** ALERT ***: Inspection scheduled for non-existent vessel!"); //someone's getting fired again
					else if(inspected.getCargo().contains("smuggle"))
					{
						System.out.println("*** ALERT ***: Vessel "+inspected.getLicense()+" contains illegal cargo: "+inspected.getCargo()+". Detaining...");
						spacePort.remove(license); //impoud the ship
					}	
				}
			} //end while
			sc.close();
		}
		catch(IOException e) { //if something goes wrong with file parsing
			e.printStackTrace();
		}
		
		//print out final information about the simulation
		System.out.println("\n\n####### SIMULATION STATISTICS #######");
		System.out.println("SpacePort: "+spacePort); //spaceport details
		
		HashMap<Statistic, Integer> stats = spacePort.getStatistics(); 
		
		System.out.println(" Current # ships docked: "+stats.get(Statistic.CURRENT_DOCKED));
		System.out.println(" Maximum # ships docked: "+stats.get(Statistic.MAX_DOCKED));
		System.out.println(" Current # of berths:    "+stats.get(Statistic.BERTHS));
		System.out.println(" Avg # collisions (successful):   "+stats.get(Statistic.AVG_COLLISIONS_SUCCESSFUL));
		System.out.println(" Max # collisions (successful):   "+stats.get(Statistic.MAX_COLLISIONS_SUCCESSFUL));
		System.out.println(" Avg # collisions (unsuccessful): "+stats.get(Statistic.AVG_COLLISIONS_UNSUCCESSFUL));
		System.out.println(" Max # collisions (unsuccessful): "+stats.get(Statistic.MAX_COLLISIONS_UNSUCCESSFUL));
		System.out.println(" Avg # collisions (all):          "+stats.get(Statistic.AVG_COLLISIONS_ALL));
		System.out.println(" Max # collisions (all):          "+stats.get(Statistic.MAX_COLLISIONS_ALL));
	}
	
	/**
	 * This method returns a specified file. File can either be specified by path in the parameter, 
	 * or if the parameter is null chosen via a JFileChooser.
	 * @param filename path for the file to return, or null to prompt the user to pick a file
	 * @return the file indicated
	 */
	public static File getFile(String filename)
	{
		File f = null; //the file we will read
		if(filename != null) //if given a filename, load that
			f = new File(filename);
		else //otherwise, pick a file
		{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new FileNameExtensionFilter("Text files", "txt")); //for choosing txt files
			int returnVal = chooser.showOpenDialog(chooser);
			if(returnVal == JFileChooser.APPROVE_OPTION) //if they picked something
				f = chooser.getSelectedFile();
		}
		return f;
	}
	
	
}
